@@ -1,4 +1,4 @@
-{-# LANGUAGE UnicodeSyntax #-}
+{-# LANGUAGE UnicodeSyntax, ExplicitForall #-}
 module UnicodeSyntax where
 
 import System.Environment (getArgs)
@@ -10,3 +10,6 @@ main = do
 
 test :: Int → Bool
 test x = x*5 == x+8
+
+id1 ∷ ∀ a . a → a
+id1 x = x
