@@ -76,25 +76,28 @@ parseFileContentsWithExts exts = parseFileContentsWithMode (defaultParseMode { e
 
 -- | Parse a source file from a string using a custom parse mode.
 parseFileContentsWithMode :: ParseMode -> String -> ParseResult Module
-parseFileContentsWithMode p@(ParseMode fn exts ign _ _) rawStr =
+parseFileContentsWithMode p@(ParseMode fn oldLang exts ign _ _) rawStr =
         let md = delit fn $ ppContents rawStr
-            allExts = -- impliesExts $ 
-                            case (ign, readExtensions md) of
-                                     (False,Just es) -> exts ++ es
-                                     _               -> exts
-         in parseWithMode (p { extensions = allExts }) md
+            (bLang, extraExts) = 
+                case (ign, readExtensions md) of
+                  (False, Just (mLang, es)) -> 
+                       (case mLang of {Nothing -> oldLang;Just newLang -> newLang}, es)
+                  _ -> (oldLang, [])
+         in parseWithMode (p { baseLanguage = bLang, extensions = exts ++ extraExts }) md
 
 -- | Parse a source file from a string using a custom parse mode and retaining comments.
 parseFileContentsWithComments :: ParseMode -> String -> ParseResult (Module, [Comment])
-parseFileContentsWithComments p@(ParseMode fn exts ign _ _) rawStr =
+parseFileContentsWithComments p@(ParseMode fn oldLang exts ign _ _) rawStr =
         let md = delit fn $ ppContents rawStr
-            allExts = impliesExts $ case (ign, readExtensions md) of
-                                     (False,Just es) -> exts ++ es
-                                     _               -> exts
-         in parseWithComments (p { extensions = allExts }) md
+            (bLang, extraExts) = 
+                case (ign, readExtensions md) of
+                  (False, Just (mLang, es)) -> 
+                       (case mLang of {Nothing -> oldLang;Just newLang -> newLang}, es)
+                  _ -> (oldLang, [])
+         in parseWithComments (p { baseLanguage = bLang, extensions = exts ++ extraExts }) md
 
 
--- | Gather the extensions declared in LANGUAGE pragmas
+{-- | Gather the extensions declared in LANGUAGE pragmas
 --   at the top of the file. Returns 'Nothing' if the
 --   parse of the pragmas fails.
 readExtensions :: String -> Maybe [Extension]
@@ -105,7 +108,32 @@ readExtensions str = case getTopPragmas str of
         getExts (LanguagePragma _ ns) = map readExt ns
         getExts _ = []
 
-        readExt (Ident e) = classifyExtension e
+        readExt (Ident e) = classifyExtension e -}
+
+-- | Gather the extensions declared in LANGUAGE pragmas
+--   at the top of the file. Returns 'Nothing' if the
+--   parse of the pragmas fails.
+readExtensions :: String -> Maybe (Maybe Language, [Extension])
+readExtensions str = case getTopPragmas str of
+        ParseOk pgms -> extractLang $ concatMap getExts pgms
+        _            -> Nothing
+  where getExts :: ModulePragma -> [Either Language Extension]
+        getExts (LanguagePragma _ ns) = map readExt ns
+        getExts _ = []
+
+        readExt (Ident e) = 
+            case classifyLanguage e of
+              UnknownLanguage _ -> Right $ classifyExtension e
+              lang -> Left lang
+
+        extractLang = extractLang' Nothing []
+
+        extractLang' lacc eacc [] = Just (lacc, eacc)
+        extractLang' Nothing eacc (Left l : rest) = extractLang' (Just l) eacc rest
+        extractLang' (Just l1) eacc (Left l2:rest)
+            | l1 == l2  = extractLang' (Just l1) eacc rest
+            | otherwise = Nothing
+        extractLang' lacc eacc (Right ext : rest) = extractLang' lacc (ext:eacc) rest
 
 ppContents :: String -> String
 ppContents = unlines . f . lines
