@@ -166,14 +166,15 @@ pList' [] _ = []
 pList' [p] (_,c) = [(p,c)]
 pList' (p:ps) (b,c) = (p, b) : pList' ps (b,c)
 
-parenList, squareList, curlyList :: (Annotated ast, ExactP ast) => [SrcSpan] -> [ast SrcSpanInfo] -> EP ()
+parenList, squareList, curlyList, parenHashList :: (Annotated ast, ExactP ast) => [SrcSpan] -> [ast SrcSpanInfo] -> EP ()
 parenList = bracketList ("(",",",")")
 squareList = bracketList ("[",",","]")
 curlyList = bracketList ("{",",","}")
+parenHashList = bracketList ("(#",",","#)")
 
 layoutList :: (Functor ast, Show (ast ()), Annotated ast, ExactP ast) => [SrcSpan] -> [ast SrcSpanInfo] -> EP ()
-layoutList poss asts = printStreams 
-        (map (pos *** printString) $ lList poss) 
+layoutList poss asts = printStreams
+        (map (pos *** printString) $ lList poss)
         (map (pos . ann &&& exactP) asts)
 
 lList (p:ps) = (if isNullSpan p then (p,"") else (p,"{")) : lList' ps
@@ -185,7 +186,7 @@ printSemi :: SrcSpan -> EP ()
 printSemi p = do
   printWhitespace (pos p)
   when (not $ isNullSpan p) $ printString ";"
-                
+
 
 --------------------------------------------------
 -- Exact printing
@@ -433,7 +434,7 @@ instance ExactP Module where
          [a,b,c,d,e] -> do
             printStreams (map (\(p,s) -> (pos p, printString s)) $ lList dPts)
                          (map (\i -> (pos $ ann i, exactPC i)) ids ++ map (\d -> (pos $ ann d, exactPC d)) (sepFunBinds decls))
-    
+
             printStringAt (pos a) "<"
             exactPC xn
             mapM_ exactPC attrs
@@ -522,7 +523,7 @@ instance ExactP Decl where
         exactPC dh
         pts <- case mk of
                 Nothing -> return pts
-                Just kd -> case pts of 
+                Just kd -> case pts of
                             p:pts' -> do
                                printStringAt (pos p) "::"
                                exactPC kd
@@ -576,7 +577,7 @@ instance ExactP Decl where
                 printStringAt (pos x) "where"
                 layoutList pts gds
                 maybeEP exactPC mder
-             _ -> errorEP "ExactP: Decl: GDataInsDecl is given too few srcInfoPoints" 
+             _ -> errorEP "ExactP: Decl: GDataInsDecl is given too few srcInfoPoints"
          _ -> errorEP "ExactP: Decl: GDataInsDecl is given too few srcInfoPoints"
     ClassDecl    l mctxt dh fds mcds    ->
         case srcInfoPoints l of
@@ -596,7 +597,7 @@ instance ExactP Decl where
                     printStringAt (pos p) "where"
                     layoutList pts' $ sepClassFunBinds cds
                  _ -> errorEP "ExactP: Decl: ClassDecl is given too few srcInfoPoints"
-                ) mcds                
+                ) mcds
          _ -> errorEP "ExactP: Decl: ClassDecl is given too few srcInfoPoints"
     InstDecl     l mctxt ih mids        ->
         case srcInfoPoints l of
@@ -665,7 +666,7 @@ instance ExactP Decl where
             maybeEP exactPC msf
             pts <- case mstr of
                       Nothing -> return pts
-                      Just str -> case pts of 
+                      Just str -> case pts of
                                    x:pts' -> do
                                       printStringAt (pos x) (show str)
                                       return pts'
@@ -734,12 +735,13 @@ instance ExactP Decl where
             exactPC qn
             printStringAt (pos b) "#-}"
          _ -> errorEP "ExactP: Decl: InlineConlikeSig is given wrong number of srcInfoPoints"
-    SpecSig          l qn ts        ->
+    SpecSig          l mact qn ts ->
         case srcInfoPoints l of
          a:pts -> do
             printString "{-# SPECIALISE"
+            maybeEP exactPC mact
             exactPC qn
-            printInterleaved (zip pts ("::" : repeat "," ++ ["#-}"])) ts
+            printInterleaved (zip pts ("::" : replicate (length pts - 2) "," ++ ["#-}"])) ts
          _ -> errorEP "ExactP: Decl: SpecSig is given too few srcInfoPoints"
     SpecInlineSig    l b mact qn ts ->
         case srcInfoPoints l of
@@ -747,7 +749,7 @@ instance ExactP Decl where
             printString $ "{-# SPECIALISE " ++ if b then "INLINE" else "NOINLINE"
             maybeEP exactPC mact
             exactPC qn
-            printInterleaved (zip pts ("::" : repeat "," ++ ["#-}"])) ts
+            printInterleaved (zip pts ("::" : replicate (length pts - 2) "," ++ ["#-}"])) ts
          _ -> errorEP "ExactP: Decl: SpecInlineSig is given too few srcInfoPoints"
     InstSig          l mctxt ih     ->
         case srcInfoPoints l of
@@ -852,7 +854,7 @@ instance ExactP Kind where
             exactPC kd
             printStringAt (pos b) ")"
          _ -> errorEP "ExactP: Kind: KindParen is given wrong number of srcInfoPoints"
-    KindVar l n     -> exactP n       
+    KindVar l n     -> exactP n
 
 instance ExactP Type where
   exactP t = case t of
@@ -878,11 +880,9 @@ instance ExactP Type where
             exactPC t2
          _ -> errorEP "ExactP: Type: TyFun is given wrong number of srcInfoPoints"
     TyTuple l bx ts -> do
-        let pts = srcInfoPoints l
-            (o,e) = case bx of
-                     Boxed   -> ("(" , ")")
-                     Unboxed -> ("(#","#)")
-        printInterleaved (zip pts (o: replicate (length pts - 2) "," ++ [e])) ts
+        case bx of
+          Boxed   -> parenList (srcInfoPoints l) ts
+          Unboxed -> parenHashList (srcInfoPoints l) ts
     TyList  l t     -> do
         case srcInfoPoints l of
          [a,b] -> do
@@ -1031,7 +1031,7 @@ instance ExactP InstDecl where
 --            printString $ if inl then "{-# INLINE" else "{-# NOINLINE"
 --            maybeEP exactPC mact
 --            exactPC qn
---            printStringAt (pos b) "#-}" 
+--            printStringAt (pos b) "#-}"
 --         _ -> errorEP "ExactP: InstDecl: InsInline is given wrong number of srcInfoPoints"
 
 instance ExactP FunDep where
@@ -1134,7 +1134,7 @@ instance ExactP Exp where
             case rest2 of
               (c:rest3) -> do
                 let (mpSemi2,rest4) = if snd (spanSize c) == 4 -- this is "else", not a semi
-                                       then (Nothing, rest2) 
+                                       then (Nothing, rest2)
                                        else (Just c, rest3)
                 case rest4 of
                   [pElse] -> do
@@ -1172,10 +1172,14 @@ instance ExactP Exp where
             printString "mdo"
             layoutList pts stmts
          _ -> errorEP "ExactP: Exp: Mdo is given wrong number of srcInfoPoints"
-    Tuple l es      -> parenList (srcInfoPoints l) es
-    TupleSection l mexps    -> do
+    Tuple l bx es   ->
+        case bx of
+          Boxed   -> parenList (srcInfoPoints l) es
+          Unboxed -> parenHashList (srcInfoPoints l) es
+    TupleSection l bx mexps -> do
         let pts = srcInfoPoints l
-        printSeq $ interleave (zip (map pos $ init pts) (map printString ("(": repeat ",")) ++ [(pos $ last pts, printString ")")])
+            (o, e) = case bx of Boxed -> ("(", ")"); Unboxed -> ("(#", "#)")
+        printSeq $ interleave (zip (map pos $ init pts) (map printString (o: repeat ",")) ++ [(pos $ last pts, printString e)])
                               (map (maybe (0,0) (pos . ann) &&& maybeEP exactPC) mexps)
     List l es               -> squareList (srcInfoPoints l) es
     Paren l p               -> parenList (srcInfoPoints l) [p]
@@ -1516,7 +1520,7 @@ instance ExactP Match where
         (cPars,whPt) = splitAt pars cParsWh    -- whPt is either singleton or empty
     printStrs (zip oPars (repeat "("))
     exactPC n
-    printStreams (zip (map pos cPars) (repeat $ printString ")")) (map (pos . ann &&& exactPC) ps) 
+    printStreams (zip (map pos cPars) (repeat $ printString ")")) (map (pos . ann &&& exactPC) ps)
     exactPC rhs
     maybeEP (\bds -> printStringAt (pos (head pts)) "where" >> exactPC bds) mbinds
   exactP (InfixMatch l a n bs rhs mbinds) = do
@@ -1559,7 +1563,10 @@ instance ExactP Pat where
          _ -> errorEP "ExactP: Pat: PNPlusK is given wrong number of srcInfoPoints"
     PInfixApp l pa qn pb -> exactP pa >> epInfixQName qn >> exactPC pb
     PApp l qn ps    -> exactP qn >> mapM_ exactPC ps
-    PTuple l ps -> parenList (srcInfoPoints l) ps
+    PTuple l bx ps ->
+        case bx of
+          Boxed   -> parenList (srcInfoPoints l) ps
+          Unboxed -> parenHashList (srcInfoPoints l) ps
     PList l ps  -> squareList (srcInfoPoints l) ps
     PParen l p  -> parenList (srcInfoPoints l) [p]
     PRec l qn pfs   -> exactP qn >> curlyList (srcInfoPoints l) pfs
@@ -1731,10 +1738,12 @@ instance ExactP CallConv where
   exactP (DotNet    _) = printString "dotnet"
   exactP (Jvm       _) = printString "jvm"
   exactP (Js        _) = printString "js"
+  exactP (CApi      _) = printString "capi"
 
 instance ExactP Safety where
   exactP (PlayRisky _) = printString "unsafe"
   exactP (PlaySafe _ b) = printString $ if b then "threadsafe" else "safe"
+  exactP (PlayInterruptible _) = printString "interruptible"
 
 instance ExactP Rule where
   exactP (Rule l str mact mrvs e1 e2) =
