@@ -11,12 +11,12 @@
 -- This module contains code for translating from the annotated
 -- complex AST in Language.Haskell.Exts.Annotated.Syntax
 -- to the simpler, sparsely annotated AST in Language.Haskell.Exts.Syntax.
--- 
+--
 -- A function @sXYZ@ translates an annotated AST node of type @XYZ l@ into
 -- a simple AST node of type @XYZ@. I would have prefered to use a MPTC
 -- with an fd/type family to get a single exported function name, but
 -- I wish to stay Haskell 2010 compliant. Let's hope for Haskell 2011.
--- 
+--
 -----------------------------------------------------------------------------
 module Language.Haskell.Exts.Annotated.Simplify where
 
@@ -103,7 +103,7 @@ sDecl decl = case decl of
      WarnPragmaDecl   l nsstrs  -> S.WarnPragmaDecl (getPointLoc l) (map (\(ns, str) -> (map sName ns, str)) nsstrs)
      InlineSig        l b mact qn   -> S.InlineSig (getPointLoc l) b (maybe S.AlwaysActive sActivation mact) (sQName qn)
      InlineConlikeSig l   mact qn   -> S.InlineConlikeSig (getPointLoc l) (maybe S.AlwaysActive sActivation mact) (sQName qn)
-     SpecSig          l qn ts   -> S.SpecSig (getPointLoc l) (sQName qn) (map sType ts)
+     SpecSig          l   mact qn ts   -> S.SpecSig (getPointLoc l) (maybe S.AlwaysActive sActivation mact) (sQName qn) (map sType ts)
      SpecInlineSig    l b mact qn ts    ->
         S.SpecInlineSig (getPointLoc l) b (maybe S.AlwaysActive sActivation mact) (sQName qn) (map sType ts)
      InstSig          l mctxt ih    ->
@@ -346,8 +346,8 @@ sExp e = case e of
     Case _ e alts       -> S.Case (sExp e) (map sAlt alts)
     Do _ ss             -> S.Do (map sStmt ss)
     MDo _ ss            -> S.MDo (map sStmt ss)
-    Tuple _ es          -> S.Tuple (map sExp es)
-    TupleSection _ mes  -> S.TupleSection (map (fmap sExp) mes)
+    Tuple _ bx es       -> S.Tuple bx (map sExp es)
+    TupleSection _ bx mes -> S.TupleSection bx (map (fmap sExp) mes)
     List _ es           -> S.List (map sExp es)
     Paren _ e           -> S.Paren (sExp e)
     LeftSection _ e op  -> S.LeftSection (sExp e) (sQOp op)
@@ -402,6 +402,7 @@ sSplice (ParenSplice _ e) = S.ParenSplice (sExp e)
 sSafety :: Safety l -> S.Safety
 sSafety (PlayRisky _) = S.PlayRisky
 sSafety (PlaySafe _ b) = S.PlaySafe b
+sSafety (PlayInterruptible _) = S.PlayInterruptible
 
 sCallConv :: CallConv l -> S.CallConv
 sCallConv (StdCall   _) = S.StdCall
@@ -409,7 +410,8 @@ sCallConv (CCall     _) = S.CCall
 sCallConv (CPlusPlus _) = S.CPlusPlus
 sCallConv (DotNet    _) = S.DotNet
 sCallConv (Jvm       _) = S.Jvm
-sCallConv (Js        _ ) = S.Js
+sCallConv (Js        _) = S.Js
+sCallConv (CApi      _) = S.CApi
 
 sModulePragma :: SrcInfo loc => ModulePragma loc -> S.ModulePragma
 sModulePragma pr = case pr of
@@ -442,7 +444,7 @@ sPat pat = case pat of
     PNPlusK _ n k       -> S.PNPlusK (sName n) k
     PInfixApp _ pa qn pb -> S.PInfixApp (sPat pa) (sQName qn) (sPat pb)
     PApp _ qn ps        -> S.PApp (sQName qn) (map sPat ps)
-    PTuple _ ps         -> S.PTuple (map sPat ps)
+    PTuple _ bx ps      -> S.PTuple bx (map sPat ps)
     PList _ ps          -> S.PList (map sPat ps)
     PParen _ p          -> S.PParen (sPat p)
     PRec _ qn pfs       -> S.PRec (sQName qn) (map sPatField pfs)
