@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, DeriveDataTypeable #-}
+{-# LANGUAGE CPP, DeriveDataTypeable, DeriveFoldable, DeriveTraversable #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.Exts.Annotated.Syntax
@@ -90,7 +90,7 @@ module Language.Haskell.Exts.Annotated.Syntax (
     unit_con, tuple_con, unboxed_singleton_con,
     -- ** Special identifiers
     as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name,
-    export_name, safe_name, unsafe_name, threadsafe_name, 
+    export_name, safe_name, unsafe_name, threadsafe_name,
     stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name,
     forall_name, family_name,
     -- ** Type constructors
@@ -113,11 +113,13 @@ import Data.Generics (Data(..),Typeable(..))
 #endif
 #endif
 
+import Data.Foldable (Foldable)
+import Data.Traversable (Traversable)
 
 -- | The name of a Haskell module.
 data ModuleName l = ModuleName l String
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -134,7 +136,7 @@ data SpecialCon l
     | Cons l                -- ^ list data constructor @(:)@
     | UnboxedSingleCon l    -- ^ unboxed singleton tuple constructor @(\# \#)@
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -146,7 +148,7 @@ data QName l
     | UnQual  l                (Name l) -- ^ unqualified local name
     | Special l (SpecialCon l)          -- ^ built-in constructor with special syntax
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -156,7 +158,7 @@ data Name l
     = Ident  l String   -- ^ /varid/ or /conid/.
     | Symbol l String   -- ^ /varsym/ or /consym/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -166,7 +168,7 @@ data IPName l
     = IPDup l String -- ^ ?/ident/, non-linear implicit parameter
     | IPLin l String -- ^ %/ident/, linear implicit parameter
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -176,7 +178,7 @@ data QOp l
     = QVarOp l (QName l) -- ^ variable operator (/qvarop/)
     | QConOp l (QName l) -- ^ constructor operator (/qconop/)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -186,7 +188,7 @@ data Op l
     = VarOp l (Name l)    -- ^ variable operator (/varop/)
     | ConOp l (Name l)    -- ^ constructor operator (/conop/)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -197,7 +199,7 @@ data CName l
     = VarName l (Name l) -- ^ name of a method or field
     | ConName l (Name l) -- ^ name of a data constructor
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -213,7 +215,7 @@ data Module l
                 (XName l) [XAttr l] (Maybe (Exp l)) [Exp l]
     -- ^ a hybrid module combining an XML document with an ordinary module
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -221,7 +223,7 @@ data Module l
 -- | The head of a module, including the name and export specification.
 data ModuleHead l = ModuleHead l (ModuleName l) (Maybe (WarningText l)) (Maybe (ExportSpecList l))
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -230,7 +232,7 @@ data ModuleHead l = ModuleHead l (ModuleName l) (Maybe (WarningText l)) (Maybe (
 data ExportSpecList l
     = ExportSpecList l [ExportSpec l]
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -250,7 +252,7 @@ data ExportSpec l
      | EModuleContents l (ModuleName l) -- ^ @module M@:
                                         --   re-export a module.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -267,7 +269,7 @@ data ImportDecl l = ImportDecl
             -- ^ optional list of import specifications.
     }
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -279,7 +281,7 @@ data ImportSpecList l
             -- The 'Bool' is 'True' if the names are excluded
             -- by @hiding@.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -297,7 +299,7 @@ data ImportSpec l
                                         --   a class imported with some of its methods, or
                                         --   a datatype imported with some of its constructors.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -308,7 +310,7 @@ data Assoc l
      | AssocLeft  l -- ^ left-associative operator (declared with @infixl@).
      | AssocRight l -- ^ right-associative operator (declared with @infixr@)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -363,7 +365,7 @@ data Decl l
      -- ^ An INLINE pragma
      | InlineConlikeSig l      (Maybe (Activation l)) (QName l)
      -- ^ An INLINE CONLIKE pragma
-     | SpecSig          l                             (QName l) [Type l]
+     | SpecSig          l      (Maybe (Activation l)) (QName l) [Type l]
      -- ^ A SPECIALISE pragma
      | SpecInlineSig    l Bool (Maybe (Activation l)) (QName l) [Type l]
      -- ^ A SPECIALISE INLINE pragma
@@ -372,7 +374,7 @@ data Decl l
      | AnnPragma        l (Annotation l)
      -- ^ An ANN pragma
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -386,7 +388,7 @@ data Annotation l
     | ModuleAnn l           (Exp l)
     -- ^ An annotation for the defining module.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -395,7 +397,7 @@ data Annotation l
 -- | A flag stating whether a declaration is a data or newtype declaration.
 data DataOrNew l = DataType l | NewType l
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -406,7 +408,7 @@ data DeclHead l
     | DHInfix l (TyVarBind l) (Name l) (TyVarBind l)
     | DHParen l (DeclHead l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -417,7 +419,7 @@ data InstHead l
     | IHInfix l (Type l) (QName l) (Type l)
     | IHParen l (InstHead l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -425,7 +427,7 @@ data InstHead l
 -- | A deriving clause following a data type declaration.
 data Deriving l = Deriving l [InstHead l]
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -435,7 +437,7 @@ data Binds l
     = BDecls  l [Decl l]     -- ^ An ordinary binding group
     | IPBinds l [IPBind l]   -- ^ A binding group for implicit parameters
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -443,7 +445,7 @@ data Binds l
 -- | A binding of an implicit parameter.
 data IPBind l = IPBind l (IPName l) (Exp l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -451,7 +453,7 @@ data IPBind l = IPBind l (IPName l) (Exp l)
 -- | Clauses of a function binding.
 data Match l
      = Match l      (Name l) [Pat l]         (Rhs l) {-where-} (Maybe (Binds l))
-        -- ^ A clause defined with prefix notation, i.e. the function name 
+        -- ^ A clause defined with prefix notation, i.e. the function name
         --  followed by its argument patterns, the right-hand side and an
         --  optional where clause.
      | InfixMatch l (Pat l) (Name l) [Pat l] (Rhs l) {-where-} (Maybe (Binds l))
@@ -461,7 +463,7 @@ data Match l
         --  Note that there can be more than two arguments to a function declared
         --  infix, hence the list of pattern arguments.
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -473,7 +475,7 @@ data QualConDecl l
         {-forall-} (Maybe [TyVarBind l]) {- . -} (Maybe (Context l))
         {- => -} (ConDecl l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -487,7 +489,7 @@ data ConDecl l
      | RecDecl l (Name l) [FieldDecl l]
                 -- ^ record constructor
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -495,7 +497,7 @@ data ConDecl l
 -- | Declaration of a (list of) named field(s).
 data FieldDecl l = FieldDecl l [Name l] (BangType l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -505,7 +507,7 @@ data FieldDecl l = FieldDecl l [Name l] (BangType l)
 data GadtDecl l
     = GadtDecl l (Name l) (Type l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -521,7 +523,7 @@ data ClassDecl l
     | ClsTyDef   l (Type l) (Type l)
             -- ^ default choice for an associated type synonym
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -537,7 +539,7 @@ data InstDecl l
     | InsGData  l (DataOrNew l) (Type l) (Maybe (Kind l)) [GadtDecl l] (Maybe (Deriving l))
             -- ^ an associated data type implemented using GADT style
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -549,7 +551,7 @@ data BangType l
      | UnBangedTy l (Type l) -- ^ non-strict component
      | UnpackedTy l (Type l) -- ^ unboxed component, marked with an UNPACK pragma
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -560,7 +562,7 @@ data Rhs l
      | GuardedRhss  l [GuardedRhs l]
                 -- ^ guarded right hand side (/gdrhs/)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -571,7 +573,7 @@ data Rhs l
 data GuardedRhs l
      = GuardedRhs l [Stmt l] (Exp l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -593,7 +595,7 @@ data Type l
      | TyInfix l (Type l) (QName l) (Type l)    -- ^ infix type constructor
      | TyKind  l (Type l) (Kind l)              -- ^ type with explicit kind signature
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -611,7 +613,7 @@ data TyVarBind l
     = KindedVar   l (Name l) (Kind l)  -- ^ variable binding with kind annotation
     | UnkindedVar l (Name l)           -- ^ ordinary variable binding
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -624,7 +626,7 @@ data Kind l
     | KindParen l (Kind l)           -- ^ a parenthesised kind
     | KindVar   l (Name l)           -- ^ a kind variable (as-of-yet unsupported by compilers)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -635,7 +637,7 @@ data Kind l
 data FunDep l
     = FunDep l [Name l] [Name l]
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -647,7 +649,7 @@ data Context l
     | CxParen  l (Context l)
     | CxEmpty  l
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -662,7 +664,7 @@ data Asst l
         | IParam l (IPName l) (Type l)          -- ^ implicit parameter assertion
         | EqualP l (Type l) (Type l)            -- ^ type equality constraint
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -683,7 +685,7 @@ data Literal l
     | PrimChar   l Char     String     -- ^ unboxed character literal
     | PrimString l String   String     -- ^ unboxed string literal
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -705,8 +707,8 @@ data Exp l
                                             --   the last statement in the list
                                             --   should be an expression.
     | MDo l [Stmt l]                        -- ^ @mdo@-expression
-    | Tuple l [Exp l]                       -- ^ tuple expression
-    | TupleSection l [Maybe (Exp l)]        -- ^ tuple section expression, e.g. @(,,3)@
+    | Tuple l Boxed [Exp l]                 -- ^ tuple expression
+    | TupleSection l Boxed [Maybe (Exp l)]  -- ^ tuple section expression, e.g. @(,,3)@
     | List l [Exp l]                        -- ^ list expression
     | Paren l (Exp l)                       -- ^ parenthesised expression
     | LeftSection l (Exp l) (QOp l)         -- ^ left section @(@/exp/ /qop/@)@
@@ -739,7 +741,7 @@ data Exp l
                                             -- ^ empty xml element, with attributes
     | XPcdata l String                      -- ^ PCDATA child element
     | XExpTag l (Exp l)                     -- ^ escaped haskell expression inside xml
-    | XChildTag l [Exp l]                   -- ^ children of an xml element    
+    | XChildTag l [Exp l]                   -- ^ children of an xml element
 
 
 -- Pragmas
@@ -755,7 +757,7 @@ data Exp l
     | LeftArrHighApp  l (Exp l) (Exp l)     -- ^ higher-order arrow application (from left): /exp/ @-<<@ /exp/
     | RightArrHighApp l (Exp l) (Exp l)     -- ^ higher-order arrow application (from right): /exp/ @>>-@ /exp/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -766,7 +768,7 @@ data XName l
     = XName l String              -- <name ...
     | XDomName l String String    -- <dom:name ...
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -774,7 +776,7 @@ data XName l
 -- | An xml attribute, which is a name-expression pair.
 data XAttr l = XAttr l (XName l) (Exp l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -786,7 +788,7 @@ data Bracket l
     | TypeBracket l (Type l)      -- ^ type bracket: @[t| ... |]@
     | DeclBracket l [Decl l]      -- ^ declaration bracket: @[d| ... |]@
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -796,7 +798,7 @@ data Splice l
     = IdSplice l String           -- ^ variable splice: @$var@
     | ParenSplice l (Exp l)       -- ^ parenthesised expression splice: @$(/exp/)@
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -805,8 +807,9 @@ data Splice l
 data Safety l
     = PlayRisky l         -- ^ unsafe
     | PlaySafe l Bool     -- ^ safe ('False') or threadsafe ('True')
+    | PlayInterruptible l -- ^ interruptible
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -819,8 +822,9 @@ data CallConv l
     | DotNet l
     | Jvm l
     | Js l
+    | CApi l
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -833,7 +837,7 @@ data ModulePragma l
     | AnnModulePragma  l (Annotation l)
                         -- ^ ANN pragma with module scope
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -851,7 +855,7 @@ data Activation l
     = ActiveFrom   l Int
     | ActiveUntil  l Int
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -860,7 +864,7 @@ data Activation l
 data Rule l
     = Rule l String (Maybe (Activation l)) (Maybe [RuleVar l]) (Exp l) (Exp l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -870,7 +874,7 @@ data RuleVar l
     = RuleVar l (Name l)
     | TypedRuleVar l (Name l) (Type l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -881,7 +885,7 @@ data WarningText l
     = DeprText l String
     | WarnText l String
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -895,7 +899,7 @@ data Pat l
     | PNPlusK l (Name l) Integer            -- ^ n+k pattern
     | PInfixApp l (Pat l) (QName l) (Pat l) -- ^ pattern with an infix data constructor
     | PApp l (QName l) [Pat l]              -- ^ data constructor and argument patterns
-    | PTuple l [Pat l]                      -- ^ tuple pattern
+    | PTuple l Boxed [Pat l]                -- ^ tuple pattern
     | PList l [Pat l]                       -- ^ list pattern
     | PParen l (Pat l)                      -- ^ parenthesized pattern
     | PRec l (QName l) [PatField l]         -- ^ labelled pattern, record style
@@ -916,7 +920,7 @@ data Pat l
     | PQuasiQuote l String String           -- ^ quasi quote pattern: @[$/name/| /string/ |]@
     | PBangPat l (Pat l)                    -- ^ strict (bang) pattern: @f !x = ...@
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -924,7 +928,7 @@ data Pat l
 -- | An XML attribute in a pattern.
 data PXAttr l = PXAttr l (XName l) (Pat l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -938,7 +942,7 @@ data RPatOp l
     | RPOpt   l  -- ^ @?@ = 0 or 1
     | RPOptG  l  -- ^ @?!@ = 0 or 1, greedy
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -954,7 +958,7 @@ data RPat l
     | RPParen l (RPat l)           -- ^ parenthesised pattern, e.g. (2*)
     | RPPat l (Pat l)              -- ^ an ordinary pattern
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -965,7 +969,7 @@ data PatField l
     | PFieldPun l (Name l)              -- ^ record field pun
     | PFieldWildcard l                  -- ^ record field wildcard
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -983,7 +987,7 @@ data Stmt l
     | LetStmt l (Binds l)   -- ^ local bindings
     | RecStmt l [Stmt l]    -- ^ a recursive binding group for arrows
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -999,7 +1003,7 @@ data QualStmt l
     | GroupUsing   l (Exp l)          -- ^ @then@ @group@ @using@ /exp/
     | GroupByUsing l (Exp l) (Exp l)  -- ^ @then@ @group@ @by@ /exp/ @using@ /exp/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -1010,7 +1014,7 @@ data FieldUpdate l
     | FieldPun l (Name l)                -- ^ record field pun
     | FieldWildcard l                    -- ^ record field wildcard
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -1019,7 +1023,7 @@ data FieldUpdate l
 data Alt l
     = Alt l (Pat l) (GuardedAlts l) (Maybe (Binds l))
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -1031,7 +1035,7 @@ data GuardedAlts l
     = UnGuardedAlt l (Exp l)         -- ^ @->@ /exp/
     | GuardedAlts  l [GuardedAlt l]  -- ^ /gdpat/
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -1040,7 +1044,7 @@ data GuardedAlts l
 data GuardedAlt l
     = GuardedAlt l [Stmt l] (Exp l)
 #ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable)
 #else
   deriving (Eq,Ord,Show)
 #endif
@@ -1085,8 +1089,8 @@ bang_name      l = Symbol l "!"
 dot_name       l = Symbol l "."
 star_name      l = Symbol l "*"
 
-export_name, safe_name, unsafe_name, threadsafe_name, 
-  stdcall_name, ccall_name, cplusplus_name, dotnet_name, 
+export_name, safe_name, unsafe_name, threadsafe_name,
+  stdcall_name, ccall_name, cplusplus_name, dotnet_name,
   jvm_name, js_name, forall_name, family_name :: l -> Name l
 export_name     l = Ident l "export"
 safe_name       l = Ident l "safe"
@@ -1236,7 +1240,7 @@ instance Functor Decl where
         InlineSig        l b mact qn      -> InlineSig (f l) b (fmap (fmap f) mact) (fmap f qn)
         InlineConlikeSig l   mact qn      -> InlineConlikeSig (f l) (fmap (fmap f) mact) (fmap f qn)
         SpecInlineSig    l b mact qn ts   -> SpecInlineSig (f l) b (fmap (fmap f) mact) (fmap f qn) (map (fmap f) ts)
-        SpecSig          l        qn ts   -> SpecSig (f l) (fmap f qn) (map (fmap f) ts)
+        SpecSig          l   mact qn ts   -> SpecSig       (f l)   (fmap (fmap f) mact) (fmap f qn) (map (fmap f) ts)
         InstSig          l mcx ih         -> InstSig (f l) (fmap (fmap f) mcx) (fmap f ih)
         AnnPragma        l ann            -> AnnPragma (f l) (fmap f ann)
       where wp f (ns, s) = (map (fmap f) ns, s)
@@ -1386,8 +1390,8 @@ instance Functor Exp where
         Case l e alts   -> Case (f l) (fmap f e) (map (fmap f) alts)
         Do l ss         -> Do (f l) (map (fmap f) ss)
         MDo l ss        -> MDo (f l) (map (fmap f) ss)
-        Tuple l es      -> Tuple (f l) (map (fmap f) es)
-        TupleSection l mes  -> TupleSection (f l) (map (fmap (fmap f)) mes)
+        Tuple l bx es   -> Tuple (f l) bx (map (fmap f) es)
+        TupleSection l bx mes -> TupleSection (f l) bx (map (fmap (fmap f)) mes)
         List l es       -> List (f l) (map (fmap f) es)
         Paren l e       -> Paren (f l) (fmap f e)
         LeftSection l e qop     -> LeftSection (f l) (fmap f e) (fmap f qop)
@@ -1443,10 +1447,16 @@ instance Functor Splice where
 instance Functor Safety where
     fmap f (PlayRisky l) = PlayRisky (f l)
     fmap f (PlaySafe l b) = PlaySafe (f l) b
+    fmap f (PlayInterruptible l) = PlayInterruptible (f l)
 
 instance Functor CallConv where
     fmap f (StdCall l) = StdCall (f l)
     fmap f (CCall l) = CCall (f l)
+    fmap f (CPlusPlus l) = CPlusPlus (f l)
+    fmap f (DotNet l) = DotNet (f l)
+    fmap f (Jvm l) = Jvm (f l)
+    fmap f (Js l) = Js (f l)
+    fmap f (CApi l) = CApi (f l)
 
 instance Functor ModulePragma where
     fmap f (LanguagePragma   l ns) = LanguagePragma (f l) (map (fmap f) ns)
@@ -1477,7 +1487,7 @@ instance Functor Pat where
       PNPlusK l n k     -> PNPlusK (f l) (fmap f n) k
       PInfixApp l pa qn pb  -> PInfixApp (f l) (fmap f pa) (fmap f qn) (fmap f pb)
       PApp l qn ps      -> PApp (f l) (fmap f qn) (map (fmap f) ps)
-      PTuple l ps       -> PTuple (f l) (map (fmap f) ps)
+      PTuple l bx ps    -> PTuple (f l) bx (map (fmap f) ps)
       PList l ps        -> PList (f l) (map (fmap f) ps)
       PParen l p        -> PParen (f l) (fmap f p)
       PRec l qn pfs     -> PRec (f l) (fmap f qn) (map (fmap f) pfs)
@@ -1707,7 +1717,7 @@ instance Annotated Decl where
         WarnPragmaDecl   l nss          -> l
         InlineSig        l b act qn     -> l
         InlineConlikeSig l   act qn     -> l
-        SpecSig          l qn ts        -> l
+        SpecSig          l   act qn ts  -> l
         SpecInlineSig    l b act qn ts  -> l
         InstSig          l cx ih        -> l
         AnnPragma        l ann          -> l
@@ -1738,7 +1748,7 @@ instance Annotated Decl where
         WarnPragmaDecl   l nss           -> WarnPragmaDecl (f l) nss
         InlineSig        l b act qn      -> InlineSig (f l) b act qn
         InlineConlikeSig l   act qn      -> InlineConlikeSig (f l) act qn
-        SpecSig          l qn ts         -> SpecSig (f l) qn ts
+        SpecSig          l   act qn ts   -> SpecSig       (f l)   act qn ts
         SpecInlineSig    l b act qn ts   -> SpecInlineSig (f l) b act qn ts
         InstSig          l mcx ih        -> InstSig (f l) mcx ih
         AnnPragma        l ann           -> AnnPragma (f l) ann
@@ -1750,7 +1760,7 @@ instance Annotated Annotation where
     amap f (Ann     l n e) = Ann     (f l) n e
     amap f (TypeAnn l n e) = TypeAnn (f l) n e
     amap f (ModuleAnn l e) = ModuleAnn (f l) e
-    
+
 instance Annotated DataOrNew where
     ann (DataType l) = l
     ann (NewType  l) = l
@@ -1947,8 +1957,8 @@ instance Annotated Exp where
         Case l e alts   -> l
         Do l ss         -> l
         MDo l ss        -> l
-        Tuple l es      -> l
-        TupleSection l mes  -> l
+        Tuple l bx es   -> l
+        TupleSection l bx mes -> l
         List l es       -> l
         Paren l e       -> l
         LeftSection l e qop     -> l
@@ -1998,8 +2008,8 @@ instance Annotated Exp where
         Case l e alts   -> Case (f l) e alts
         Do l ss         -> Do (f l) ss
         MDo l ss        -> MDo (f l) ss
-        Tuple l es      -> Tuple (f l) es
-        TupleSection l mes  -> TupleSection (f l) mes
+        Tuple l bx es   -> Tuple (f l) bx es
+        TupleSection l bx mes -> TupleSection (f l) bx mes
         List l es       -> List (f l) es
         Paren l e       -> Paren (f l) e
         LeftSection l e qop     -> LeftSection (f l) e qop
@@ -2064,11 +2074,17 @@ instance Annotated Splice where
 instance Annotated Safety where
     ann (PlayRisky l) = l
     ann (PlaySafe l b) = l
+    ann (PlayInterruptible l) = l
     amap = fmap
 
 instance Annotated CallConv where
     ann (StdCall l) = l
     ann (CCall l) = l
+    ann (CPlusPlus l) = l
+    ann (DotNet l) = l
+    ann (Jvm l) = l
+    ann (Js l) = l
+    ann (CApi l) = l
     amap = fmap
 
 instance Annotated ModulePragma where
@@ -2107,7 +2123,7 @@ instance Annotated Pat where
       PNPlusK l n k     -> l
       PInfixApp l pa qn pb  -> l
       PApp l qn ps      -> l
-      PTuple l ps       -> l
+      PTuple l bx ps    -> l
       PList l ps        -> l
       PParen l p        -> l
       PRec l qn pfs     -> l
@@ -2132,7 +2148,7 @@ instance Annotated Pat where
       PNPlusK l n k     -> PNPlusK (f l) n k
       PInfixApp l pa qn pb  -> PInfixApp (f l) pa qn pb
       PApp l qn ps      -> PApp (f l) qn ps
-      PTuple l ps       -> PTuple (f l) ps
+      PTuple l bx ps    -> PTuple (f l) bx ps
       PList l ps        -> PList (f l) ps
       PParen l p        -> PParen (f l) p
       PRec l qn pfs     -> PRec (f l) qn pfs
