@@ -187,12 +187,14 @@ data Token
         | KW_Safe
         | KW_Unsafe
         | KW_Threadsafe
+        | KW_Interruptible
         | KW_StdCall
         | KW_CCall
         | KW_CPlusPlus
         | KW_DotNet
         | KW_Jvm
         | KW_Js
+        | KW_CApi
 
         | EOF
         deriving (Eq,Show)
@@ -285,16 +287,18 @@ special_varids = [
  ( "hiding",    (KW_Hiding,     Nothing) ),
 
 -- FFI
- ( "export",     (KW_Export,        Just (Any [ForeignFunctionInterface])) ),
- ( "safe",       (KW_Safe,          Just (Any [ForeignFunctionInterface])) ),
- ( "unsafe",     (KW_Unsafe,        Just (Any [ForeignFunctionInterface])) ),
- ( "threadsafe", (KW_Threadsafe,    Just (Any [ForeignFunctionInterface])) ),
- ( "stdcall",    (KW_StdCall,       Just (Any [ForeignFunctionInterface])) ),
- ( "ccall",      (KW_CCall,         Just (Any [ForeignFunctionInterface])) ),
- ( "cplusplus",  (KW_CPlusPlus,     Just (Any [ForeignFunctionInterface])) ),
- ( "dotnet",     (KW_DotNet,        Just (Any [ForeignFunctionInterface])) ),
- ( "jvm",        (KW_Jvm,           Just (Any [ForeignFunctionInterface])) ),
- ( "js",         (KW_Js,            Just (Any [ForeignFunctionInterface])) )
+ ( "export",        (KW_Export,        Just (Any [ForeignFunctionInterface])) ),
+ ( "safe",          (KW_Safe,          Just (Any [ForeignFunctionInterface])) ),
+ ( "unsafe",        (KW_Unsafe,        Just (Any [ForeignFunctionInterface])) ),
+ ( "threadsafe",    (KW_Threadsafe,    Just (Any [ForeignFunctionInterface])) ),
+ ( "interruptible", (KW_Interruptible, Just (Any [InterruptibleFFI])) ),
+ ( "stdcall",       (KW_StdCall,       Just (Any [ForeignFunctionInterface])) ),
+ ( "ccall",         (KW_CCall,         Just (Any [ForeignFunctionInterface])) ),
+ ( "cplusplus",     (KW_CPlusPlus,     Just (Any [ForeignFunctionInterface])) ),
+ ( "dotnet",        (KW_DotNet,        Just (Any [ForeignFunctionInterface])) ),
+ ( "jvm",           (KW_Jvm,           Just (Any [ForeignFunctionInterface])) ),
+ ( "js",            (KW_Js,            Just (Any [ForeignFunctionInterface])) ),
+ ( "capi",          (KW_CApi,          Just (Any [CApiFFI])) )
  ]
 
 pragmas :: [(String,Token)]
@@ -340,7 +344,8 @@ topLexer :: Lex a (Loc Token)
 topLexer = do
     b <- pullCtxtFlag
     if b then -- trace (show cf ++ ": " ++ show VRightCurly) $
-              setBOL >> getSrcLocL >>= \l -> return (Loc (mkSrcSpan l l) VRightCurly) -- the lex context state flags that we must do an empty {} - UGLY
+              -- the lex context state flags that we must do an empty {} - UGLY
+              setBOL >> getSrcLocL >>= \l -> return (Loc (mkSrcSpan l l) VRightCurly)
      else do
         bol <- checkBOL
         (bol, ws) <- lexWhiteSpace bol
@@ -366,7 +371,7 @@ lexWhiteSpace bol = do
     case s of
         -- If we find a recognised pragma, we don't want to treat it as a comment.
         '{':'-':'#':rest | isRecognisedPragma rest -> return (bol, False)
-                         | isLinePragma rest && not ignL -> do 
+                         | isLinePragma rest && not ignL -> do
                             (l, fn) <- lexLinePragma
                             setSrcLineL l
                             setLineFilenameL fn
@@ -584,17 +589,20 @@ lexStdToken :: Lex a Token
 lexStdToken = do
     s <- getInput
     exts <- getExtensionsL
+    let intHash = lexHash IntTok IntTokHash (Right WordTokHash)
     case s of
         [] -> return EOF
 
         '0':c:d:_ | toLower c == 'o' && isOctDigit d -> do
                         discard 2
                         (n, str) <- lexOctal
-                        return (IntTok (n, '0':c:str))
+                        con <- intHash
+                        return (con (n, '0':c:str))
                   | toLower c == 'x' && isHexDigit d -> do
                         discard 2
                         (n, str) <- lexHexadecimal
-                        return (IntTok (n, '0':c:str))
+                        con <- intHash
+                        return (con (n, '0':c:str))
 
         -- implicit parameters
         '?':c:_ | isLower c && ImplicitParams `elem` exts -> do
@@ -658,7 +666,7 @@ lexStdToken = do
         -- end template haskell
 
         -- hsx
-        '<':'%':c:_ | XmlSyntax `elem` exts -> do 
+        '<':'%':c:_ | XmlSyntax `elem` exts -> do
                         case c of
                          '>' -> do discard 3
                                    pushExtContextL ChildCtxt
@@ -962,8 +970,8 @@ lexConIdOrQual qual = do
                                               _   -> QVarSym (qual', sym)
 
           '#':cs
-            | null cs || 
-              not (isHSymbol $ head cs) && 
+            | null cs ||
+              not (isHSymbol $ head cs) &&
               not (isIdent $ head cs) && MagicHash `elem` exts -> do
                 discard 1
                 case conid of
@@ -1152,7 +1160,11 @@ parseInteger radix ds =
     foldl1 (\n d -> n * radix + d) (map (toInteger . digitToInt) ds)
 
 flagKW :: Token -> Lex a ()
-flagKW t = when (t `elem` [KW_Do, KW_MDo]) flagDo
+flagKW t = do
+  when (t `elem` [KW_Do, KW_MDo]) $ do
+       exts <- getExtensionsL
+       when (NondecreasingIndentation `elem` exts) $
+            flagDo
 
 ------------------------------------------------------------------
 -- "Pretty" printing for tokens
@@ -1291,6 +1303,7 @@ showToken t = case t of
   KW_Safe       -> "safe"
   KW_Unsafe     -> "unsafe"
   KW_Threadsafe -> "threadsafe"
+  KW_Interruptible -> "interruptible"
   KW_StdCall    -> "stdcall"
   KW_CCall      -> "ccall"
 
