@@ -137,9 +137,9 @@ checkAssertion (TyPred _ p@(IParam _ _ _)) = return p
 checkAssertion (TyPred _ p@(EqualP _ _ _)) = return p
 checkAssertion t = checkAssertion' id [] t
     where   -- class assertions must have at least one argument
-            checkAssertion' fl ts@(_:xs) (TyCon l c) = do
-                when (not $ null xs) $ checkEnabled MultiParamTypeClasses
-                when (isSymbol c)    $ checkEnabled TypeOperators
+            checkAssertion' fl ts (TyCon l c) = do
+                when (length ts /= 1) $ checkEnabled MultiParamTypeClasses
+                when (isSymbol c)     $ checkEnabled TypeOperators
                 return $ ClassA (fl l) c ts
             checkAssertion' fl ts (TyApp l a t) = do
                 -- no check on t at this stage
@@ -338,10 +338,10 @@ checkPat e [] = case e of
                             return (PNPlusK (loc <** [pp,kp]) n k)
                         _ -> patFail ""
             _ -> patFail ""
-    TupleSection l mes    ->
+    TupleSection l bx mes    ->
             if not (any ((==) Nothing) mes)
              then do ps <- mapM (\e -> checkPat e []) (map fromJust mes)
-                     return (PTuple l ps)
+                     return (PTuple l bx ps)
              else fail "Illegal tuple section in pattern"
 
     List l es      -> do
@@ -537,6 +537,8 @@ checkExpr e = case e of
     Lit l lit             -> return $ S.Lit l lit
     InfixApp l e1 op e2   -> check2Exprs e1 e2 (flip (S.InfixApp l) op)
     App l e1 e2           -> check2Exprs e1 e2 (S.App l)
+    NegApp l (Lit _ (PrimWord _ i _))
+                          -> fail $ "Parse error: negative primitive word literal: " ++ prettyPrint e
     NegApp l e            -> check1Expr e (S.NegApp l)
     Lambda loc ps e       -> check1Expr e (S.Lambda loc ps)
     Let l bs e            -> check1Expr e (S.Let l bs)
@@ -546,11 +548,11 @@ checkExpr e = case e of
                      return (S.Case l e alts)
     Do l stmts            -> checkDo stmts >> return (S.Do l stmts)
     MDo l stmts           -> checkDo stmts >> return (S.MDo l stmts)
-    TupleSection l mes -> if not (any ((==) Nothing) mes)
-                           then checkManyExprs (map fromJust mes) (S.Tuple l)
-                           else do checkEnabled TupleSections
-                                   mes' <- mapM mCheckExpr mes
-                                   return $ S.TupleSection l mes'
+    TupleSection l bx mes -> if not (any ((==) Nothing) mes)
+                             then checkManyExprs (map fromJust mes) (S.Tuple l bx)
+                             else do checkEnabled TupleSections
+                                     mes' <- mapM mCheckExpr mes
+                                     return $ S.TupleSection l bx mes'
 
 
     List l es         -> checkManyExprs es (S.List l)
@@ -693,7 +695,7 @@ checkValDef l lhs optsig rhs whereBinds = do
             case optsig of -- only pattern bindings can have signatures
                 Nothing -> return (FunBind l $
                             if b then [Match l' f ps rhs whereBinds]
-                                 else let (a:bs) = ps 
+                                 else let (a:bs) = ps
                                        in [InfixMatch l' a f bs rhs whereBinds])
                 Just _  -> fail "Cannot give an explicit type signature to a function binding"
      Nothing     -> do
@@ -714,9 +716,9 @@ isFunLhs (InfixApp ll l (QVarOp loc (UnQual _ op)) r) es
 isFunLhs (App _ (Var _ (UnQual _ f)) e) es = return $ Just (f, e:es, True, [])
 isFunLhs (App _ f e) es = isFunLhs f (e:es)
 isFunLhs (Var _ (UnQual _ f)) es@(_:_) = return $ Just (f, es, True, [])
-isFunLhs (Paren l f) es@(_:_) = do mlhs <- isFunLhs f es 
+isFunLhs (Paren l f) es@(_:_) = do mlhs <- isFunLhs f es
                                    case mlhs of
-                                    Just (f,es,b,pts) -> 
+                                    Just (f,es,b,pts) ->
                                        let [x,y] = srcInfoPoints l
                                         in return $ Just (f,es,b,x:pts++[y])
                                     _ -> return Nothing
