@@ -284,8 +284,8 @@ ppModuleHeader m mbWarn mbExportList = mySep [
         text "where"]
 
 ppWarnTxt :: WarningText -> Doc
-ppWarnTxt (DeprText s) = mySep [text "{-# DEPRECATED", text s, text "#-}"]
-ppWarnTxt (WarnText s) = mySep [text "{-# WARNING",    text s, text "#-}"]
+ppWarnTxt (DeprText s) = mySep [text "{-# DEPRECATED", text (show s), text "#-}"]
+ppWarnTxt (WarnText s) = mySep [text "{-# WARNING",    text (show s), text "#-}"]
 
 instance Pretty ModuleName where
         pretty (ModuleName modName) = text modName
@@ -481,10 +481,10 @@ instance Pretty Decl where
                 markLine pos $
                 mySep [text "{-# INLINE_CONLIKE", pretty activ, pretty name, text "#-}"]
 
-        pretty (SpecSig pos name types) =
+        pretty (SpecSig pos activ name types) =
                 blankline $
                 markLine pos $
-                mySep $ [text "{-# SPECIALISE", pretty name, text "::"]
+                mySep $ [text "{-# SPECIALISE", pretty activ, pretty name, text "::"]
                     ++ punctuate comma (map pretty types) ++ [text "#-}"]
 
         pretty (SpecInlineSig pos inl activ name types) =
@@ -587,6 +587,7 @@ instance Pretty InstDecl where
 instance Pretty Safety where
         pretty PlayRisky        = text "unsafe"
         pretty (PlaySafe b)     = text $ if b then "threadsafe" else "safe"
+        pretty PlayInterruptible = text "interruptible"
 
 instance Pretty CallConv where
         pretty StdCall   = text "stdcall"
@@ -628,7 +629,7 @@ instance Pretty ModulePragma where
         myFsep $ [text "{-# OPTIONS", text s, text "#-}"]
     pretty (AnnModulePragma _ ann) =
         myFsep $ [text "{-# ANN", pretty ann, text "#-}"]
-        
+
 
 instance Pretty Tool where
     pretty (UnknownTool s) = text s
@@ -786,16 +787,24 @@ instance Pretty Exp where
         prettyPrec p (Case cond altList) = parensIf (p > 1) $
                 myFsep [text "case", pretty cond, text "of"]
                 $$$ ppBody caseIndent (map pretty altList)
-        prettyPrec p (Do stmtList) = parensIf (p > 1) $ 
+        prettyPrec p (Do stmtList) = parensIf (p > 1) $
                 text "do" $$$ ppBody doIndent (map pretty stmtList)
-        prettyPrec p (MDo stmtList) = parensIf (p > 1) $ 
+        prettyPrec p (MDo stmtList) = parensIf (p > 1) $
                 text "mdo" $$$ ppBody doIndent (map pretty stmtList)
         -- Constructors & Vars
         prettyPrec _ (Var name) = pretty name
         prettyPrec _ (IPVar ipname) = pretty ipname
         prettyPrec _ (Con name) = pretty name
-        prettyPrec _ (Tuple expList) = parenList . map pretty $ expList
-        prettyPrec _ (TupleSection mExpList) = parenList . map (maybePP pretty) $ mExpList
+        prettyPrec _ (Tuple bxd expList) =
+                let ds = map pretty expList
+                in case bxd of
+                       Boxed   -> parenList ds
+                       Unboxed -> hashParenList ds
+        prettyPrec _ (TupleSection bxd mExpList) =
+                let ds = map (maybePP pretty) mExpList
+                in case bxd of
+                       Boxed   -> parenList ds
+                       Unboxed -> hashParenList ds
         -- weird stuff
         prettyPrec _ (Paren e) = parens . pretty $ e
         prettyPrec _ (LeftSection e op) = parens (pretty e <+> pretty op)
@@ -820,9 +829,9 @@ instance Pretty Exp where
                 bracketList ([pretty e, char '|']
                              ++ (punctuate comma . map pretty $ qualList))
         prettyPrec _ (ParComp e qualLists) =
-                bracketList (intersperse (char '|') $
-                                pretty e : (punctuate comma . concatMap (map pretty) $ qualLists))
-        prettyPrec p (ExpTypeSig _pos e ty) = parensIf (p > 0) $ 
+                bracketList (punctuate (char '|') $
+                                pretty e : (map (hsep . punctuate comma . map pretty) $ qualLists))
+        prettyPrec p (ExpTypeSig _pos e ty) = parensIf (p > 0) $
                 myFsep [pretty e, text "::", pretty ty]
         -- Template Haskell
         prettyPrec _ (BracketExp b) = pretty b
@@ -844,7 +853,7 @@ instance Pretty Exp where
                 myFsep $ [text "<%", pretty e, text "%>"]
         prettyPrec _ (XChildTag _ cs) =
                 myFsep $ text "<%>" : map pretty cs ++ [text "</%>"]
-        
+
         -- Pragmas
         prettyPrec p (CorePragma s e) = myFsep $ map text ["{-# CORE", show s, "#-}"] ++ [pretty e]
         prettyPrec _ (SCCPragma  s e) = myFsep $ map text ["{-# SCC",  show s, "#-}"] ++ [pretty e]
@@ -901,7 +910,11 @@ instance Pretty Pat where
                 myFsep [prettyPrec 1 a, pretty (QConOp op), prettyPrec 1 b]
         prettyPrec p (PApp n ps) = parensIf (p > 1 && not (null ps)) $
                 myFsep (pretty n : map (prettyPrec 2) ps)
-        prettyPrec _ (PTuple ps) = parenList . map pretty $ ps
+        prettyPrec _ (PTuple bxd ps) =
+                let ds = map pretty ps
+                in case bxd of
+                       Boxed   -> parenList ds
+                       Unboxed -> hashParenList ds
         prettyPrec _ (PList ps) =
                 bracketList . punctuate comma . map pretty $ ps
         prettyPrec _ (PParen pat) = parens . pretty $ pat
@@ -1534,7 +1547,11 @@ instance SrcInfo loc => Pretty (P.PExp loc) where
         pretty (P.Var _ name) = pretty name
         pretty (P.IPVar _ ipname) = pretty ipname
         pretty (P.Con _ name) = pretty name
-        pretty (P.TupleSection _ mExpList) = parenList . map (maybePP pretty) $ mExpList
+        pretty (P.TupleSection _ bxd mExpList) =
+                let ds = map (maybePP pretty) mExpList
+                in case bxd of
+                       Boxed   -> parenList ds
+                       Unboxed -> hashParenList ds
         pretty (P.Paren _ e) = parens . pretty $ e
         pretty (P.RecConstr _ c fieldList) =
                 pretty c <> (braceList . map pretty $ fieldList)
