@@ -86,7 +86,7 @@ module Language.Haskell.Exts.Syntax (
     unit_con, tuple_con, unboxed_singleton_con,
     -- ** Special identifiers
     as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name,
-    export_name, safe_name, unsafe_name, threadsafe_name, 
+    export_name, safe_name, unsafe_name, threadsafe_name,
     stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name,
     forall_name, family_name,
     -- ** Type constructors
@@ -330,7 +330,7 @@ data Decl
      -- ^ An INLINE pragma
      | InlineConlikeSig SrcLoc      Activation QName
      -- ^ An INLINE CONLIKE pragma
-     | SpecSig          SrcLoc                 QName [Type]
+     | SpecSig          SrcLoc      Activation QName [Type]
      -- ^ A SPECIALISE pragma
      | SpecInlineSig    SrcLoc Bool Activation QName [Type]
      -- ^ A SPECIALISE INLINE pragma
@@ -605,8 +605,8 @@ data Exp
                                 --   the last statement in the list
                                 --   should be an expression.
     | MDo [Stmt]                -- ^ @mdo@-expression
-    | Tuple [Exp]               -- ^ tuple expression
-    | TupleSection [Maybe Exp]  -- ^ tuple section expression, e.g. @(,,3)@
+    | Tuple Boxed [Exp]         -- ^ tuple expression
+    | TupleSection Boxed [Maybe Exp]  -- ^ tuple section expression, e.g. @(,,3)@
     | List [Exp]                -- ^ list expression
     | Paren Exp                 -- ^ parenthesised expression
     | LeftSection Exp QOp       -- ^ left section @(@/exp/ /qop/@)@
@@ -706,6 +706,7 @@ data Splice
 data Safety
     = PlayRisky         -- ^ unsafe
     | PlaySafe Bool     -- ^ safe ('False') or threadsafe ('True')
+    | PlayInterruptible -- ^ interruptible
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -720,6 +721,7 @@ data CallConv
     | DotNet
     | Jvm
     | Js
+    | CApi
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -789,7 +791,7 @@ data Pat
     | PNPlusK Name Integer          -- ^ n+k pattern
     | PInfixApp Pat QName Pat       -- ^ pattern with an infix data constructor
     | PApp QName [Pat]              -- ^ data constructor and argument patterns
-    | PTuple [Pat]                  -- ^ tuple pattern
+    | PTuple Boxed [Pat]            -- ^ tuple pattern
     | PList [Pat]                   -- ^ list pattern
     | PParen Pat                    -- ^ parenthesized pattern
     | PRec QName [PatField]         -- ^ labelled pattern, record style
@@ -979,7 +981,7 @@ bang_name      = Symbol "!"
 dot_name       = Symbol "."
 star_name      = Symbol "*"
 
-export_name, safe_name, unsafe_name, threadsafe_name, 
+export_name, safe_name, unsafe_name, threadsafe_name,
   stdcall_name, ccall_name, cplusplus_name, dotnet_name,
   jvm_name, js_name, forall_name, family_name :: Name
 export_name     = Ident "export"
